import numpy
import six

from loopy.target import TargetBase
from loopy.target.c.codegen.expression import LoopyCCodeMapper

_registry = {numpy.float32: 'float',
             numpy.int32 : 'int'}

def dune_function_manglers():
    # OpenCL example impls: target/opencl/__init__.py:108
    return []

def dune_symbol_manglers():
    # OpenCL example impls: target/opencl/__init__.py:137
    return []

class DuneTarget(TargetBase):

    def function_manglers(self):
        return super(DuneTarget, self).function_manglers() + dune_function_manglers()

    def symbol_manglers(self):
        return super(DuneTarget, self).symbol_manglers() + dune_symbol_manglers()

    def get_or_register_dtype(self, names, dtype=None):
        return dtype

    def dtype_to_typename(self, dtype):
        # For now, we do this the simplest possible way
        assert(type(dtype) == numpy.dtype)
        return _registry[dtype.name]
#
#     def is_vector_dtype(self, dtype):
#         raise NotImplementedError()
#
#     def vector_dtype(self, base, count):
#         raise NotImplementedError()

    def get_expression_to_code_mapper(self, codegen_state):
        return LoopyCCodeMapper(codegen_state)

    def generate_code(self, kernel, codegen_state, impl_arg_info):
        print "Somebody wants me to generate code!"
        from cgen import Block
        body = Block()

        # {{{ declare temporaries
#
#         body.extend(
#                 idi.cgen_declarator
#                 for tv in six.itervalues(kernel.temporary_variables)
#                 for idi in tv.decl_info(
#                     kernel.target,
#                     is_written=True, index_dtype=kernel.index_dtype))
#
#         # }}}

        from loopy.codegen.loop import set_up_hw_parallel_loops
        gen_code = set_up_hw_parallel_loops(kernel, 0, codegen_state)

        from cgen import Line
        body.append(Line())

        if isinstance(gen_code.ast, Block):
            body.extend(gen_code.ast.contents)
        else:
            body.append(gen_code.ast)

        return str(body), gen_code.implemented_domains
